@@ -108,9 +108,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
     flags = set()
 
     def _checkCoroutine(field):
-        """ Check only for co-routine nature of the field and only update that.
-
-        """
+        """Check only for co-routine nature of the field and only update that."""
         # TODO: This is clumsy code, trying to achieve what non-local does for
         # Python2 as well.
 
@@ -492,7 +490,7 @@ def buildFrameNode(provider, nodes, code_object, source_ref):
 
 
 def makeStatementsSequenceOrStatement(statements, source_ref):
-    """ Make a statement sequence, but only if more than one statement
+    """Make a statement sequence, but only if more than one statement
 
     Useful for when we can unroll constructs already here, but are not sure if
     we actually did that. This avoids the branch or the pollution of doing it
@@ -645,7 +643,7 @@ def makeDictCreationOrConstant(keys, values, source_ref):
 
 def makeDictCreationOrConstant2(keys, values, source_ref):
     # Create dictionary node. Tries to avoid it for constant values that are not
-    # mutable. Keys are strings.
+    # mutable. Keys are Python strings here.
 
     assert len(keys) == len(values)
     for value in values:
